﻿using Microsoft.Extensions.Logging;
using System;

namespace Demo.ConsoleApp
{
	public class SMLoggerProvider : ILoggerProvider
	{
		public ILogger CreateLogger(string categoryName)
		{
			if (categoryName == "Microsoft.EntityFrameworkCore.Database.Command")
			{
				return new SMLogger();
			}

			return new NoOpLogger();
		}

		public void Dispose()
		{ }

		private class SMLogger : ILogger
		{
			public bool IsEnabled(LogLevel logLevel)
			{
				return true;
			}

			public void Log<TState>(LogLevel logLevel, EventId eventId, TState state, Exception exception, Func<TState, Exception, string> formatter)
			{
				if (eventId.Id == Microsoft.EntityFrameworkCore.Diagnostics.RelationalEventId.CommandExecuted.Id)
					Console.WriteLine(formatter(state, exception));
			}

			public IDisposable BeginScope<TState>(TState state)
			{
				return null;
			}
		}

		private class NoOpLogger : ILogger
		{
			public bool IsEnabled(LogLevel logLevel)
			{
				return false;
			}

			public void Log<TState>(LogLevel logLevel, EventId eventId, TState state, Exception exception, Func<TState, Exception, string> formatter)
			{ }

			public IDisposable BeginScope<TState>(TState state)
			{
				return null;
			}
		}
	}
}
