﻿using Demo.ConsoleApp.Data;
using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.Infrastructure;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using System;
using System.Diagnostics;
using System.Linq;
using Z.EntityFramework.Plus;

namespace Demo.ConsoleApp
{
	class Program
	{
		private static Func<northwindContext, string, Customers> _getCustomerById =
			EF.CompileQuery((northwindContext db, string id) =>
				db.Customers.FirstOrDefault(c => c.CustomerId == id));

		static void Main(string[] args)
		{
			//SetLogging();

			//TableSplitting1();

			//TableSplitting2();

			//ModelLevelFilter();

			//StringInterpolation();

			//LikeOperator();

			//AttachGraph();

			//Future();

			//BatchUpdate();

			//CompileQuery();

			Console.ReadLine();
		}

		private static void SetLogging()
		{
			using (var ctx = new northwindContext())
			{
				var serviceProvider = ctx.GetInfrastructure();
				var loggerFactory = serviceProvider.GetService<ILoggerFactory>();
				loggerFactory.AddProvider(new SMLoggerProvider());
			}
		}

		private static void TableSplitting1()
		{
			using (var ctx = new northwindContext())
			{
				var x1 = ctx.Employees.ToList();
				var x2 = ctx.EmployeesExtended.ToList();
				var x3 = ctx.Employees.Include(c => c.EmployeeExtended).ToList();
			}
		}

		private static void TableSplitting2()
		{
			using (var ctx = new northwindContext())
			{
				var e = new Employees
				{
					Address = "a",
					BirthDate = DateTime.Now,
					City = "c",
					Country = "co",
					EmployeeExtended = new EmployeesExtended
					{
						Notes = "note",
						Photo = new byte[0]
					},
					Extension = "ext",
					FirstName = "Stefano",
					HireDate = DateTime.Now,
					HomePhone = "phn",
					LastName = "mostarda",
					PostalCode = "00100",
					Region = "Italia",
					Title = "Mr",
					TitleOfCourtesy = "Mr"
				};
				ctx.Add(e);
				ctx.SaveChanges();
			}
		}

		private static void ModelLevelFilter()
		{
			using (var ctx = new northwindContext())
			{
				var x1 = ctx.Orders.ToList();
				var x2 = ctx.OrderDetails.Include(c => c.Order).ToList();
				var x3 = ctx.OrderDetails.Select(c => new { c.OrderId, c.Order.ShipCity }).ToList();
			}
		}

		private static void StringInterpolation()
		{
			using (var ctx = new northwindContext())
			{
				var customerId = "ALFKI";
				var query = ctx.Orders.FromSql($"Select * From Orders where Customerid = {customerId}").ToList();
			}
		}

		private static void LikeOperator()
		{
			using (var ctx = new northwindContext())
			{
				var x = ctx.Employees.Where(c => EF.Functions.Like(c.FirstName, "S")).ToList();
			}
		}

		private static void AttachGraph()
		{
			using (var ctx = new northwindContext())
			{

				var o = new Orders();
				o.Customer = new Customers { CustomerId = "ALFKI" };
				o.Freight = 0;
				o.OrderDate = DateTime.Now;
				o.ShipAddress = "add";
				o.ShipCity = "cit";
				o.ShipCountry = "cou";
				o.ShipName = "nam";
				o.ShipPostalCode = "00100";
				o.ShipRegion = "Italy";
				o.OrderDetails.Add(new OrderDetails
				{
					Discount = 0,
					Quantity = 1,
					UnitPrice = 10,
					Product = new Products { ProductId = 1 }
				});
				ctx.Orders.Attach(o);
				ctx.SaveChanges();
			}
		}

		private static void Future()
		{
			using (var ctx = new northwindContext())
			{
				var countFuture = ctx.Employees.DeferredCount().FutureValue();
				var itemsFuture = ctx.Employees.OrderBy(c => c.LastName).Take(10).Future();

				var count = countFuture.Value;
				var items = itemsFuture.ToList();
			}
		}

		private static void BatchUpdate()
		{
			using (var ctx = new northwindContext())
			{
				ctx.Products.Where(c => c.SupplierId == 2).Update(p => new Products { UnitPrice = p.UnitPrice + 1 });
			}
		}

		private static void CompileQuery()
		{
			using (var db = new northwindContext())
			{
				var customer = _getCustomerById(db, "ALFKI");
			}

			var sw = new Stopwatch();
			sw.Start();
			for (var i = 0; i < 1000; i++)
			{
				using (var db = new northwindContext())
				{
					var customer = _getCustomerById(db, "ALFKI");
				}
			}
			sw.Stop();
			Console.WriteLine($"Elapsed {sw.ElapsedMilliseconds}");

			sw.Reset();
			sw.Start();
			for (var i = 0; i < 1000; i++)
			{
				using (var db = new northwindContext())
				{
					var id = "ALFKI";
					var customer = db.Customers.FirstOrDefault(c => c.CustomerId == id);
				}
			}
			sw.Stop();
			Console.WriteLine($"Elapsed {sw.ElapsedMilliseconds}");
		}
	}
}
